
#import "XS3VoicePrintView.h"
#import "dpsclient/wraps.h"
#import "XS3DSAData.h"
#import "VPWraps.h"
#import "VoicePrinter.h"


@implementation XS3VoicePrintView
- (void)setMessageTable:(NXStringTable *)aTable
{
msgTable=aTable;
}

//  function to set the current color according to the palette choosen
- (void)setLevelValue:(float) aValue :(int)myPalette
{
float f;
static NXColor myColor;
switch (myPalette)
	{
	case 0:// normal
	if ( aValue<(1.0/4.0) )
		myColor= NXConvertHSBToColor( blueValue ,1.0 , aValue/(1.0/4.0));
	else
		{
		f=blueValue+(1.0-blueValue+yellowValue)*(aValue-(1.0/4.0))/(3.0/4.0);
		if (f>1.0)
			f-=1.0;
		myColor=NXConvertHSBToColor(f ,1.0 ,1.0);
		}
	break;

	case 1:// normal with smaller range
	if ( aValue<(1.0/7.0) )
		myColor= NXConvertHSBToColor( blueValue ,1.0 , aValue/(1.0/7.0));
	else
		{
		f=blueValue+(1.0-blueValue+yellowValue)*(aValue-(1.0/7.0))/(6.0/7.0);
		if (f>1.0)
			f-=1.0;
		myColor=NXConvertHSBToColor(f ,1.0 ,1.0);
		}
	break;

	case 2:// grey level
	myColor=NXConvertGrayToColor(aValue);
	break;
	
	case 3:// grey level inversed
	myColor=NXConvertGrayToColor(1.0-aValue);
	break;

	case 4:// blue-green
	myColor=NXConvertHSBToColor(blueValue-aValue*(blueValue-greenValue),1.0 ,1.0);
	}
NXSetColor(myColor);
}

// set info strings with localized message
- (void)resetStrings
{
strcpy(channelString[0],[msgTable valueForStringKey:"MONO"]);
strcpy(channelString[1],[msgTable valueForStringKey:"STEREO"]);
strcpy(precisionString[0],[msgTable valueForStringKey:"LOW"]);
strcpy(precisionString[1],[msgTable valueForStringKey:"MEDIUM"]);
strcpy(precisionString[2],[msgTable valueForStringKey:"HIGH"]);
	return ;
}

- initFrame:(const NXRect *)frameRect
{
NXColor myColor;
float f1,f2;
[super initFrame:frameRect];
[self allocateGState];
myColor=NXConvertRGBToColor(0.0 , 0.0 , 1.0 );
NXConvertColorToHSB(myColor,&blueValue ,&f1,&f2);

myColor=NXConvertRGBToColor(0.0 , 1.0 , 0.0 );
NXConvertColorToHSB(myColor,&greenValue ,&f1,&f2);

myColor=NXConvertRGBToColor(1.0 , 1.0 , 0.0 );
NXConvertColorToHSB(myColor,&yellowValue ,&f1,&f2);
NXSetRect(&paletteRect1,80,107,120,40);
NXSetRect(&paletteRect1,80,107,120,40);
NXSetRect(&precisionRect,377,108,50,15);
precision=-1;	
selection_mode=0;
stereo=NO;
palette1=0;
strcpy(precisionString[0],"LOW");
strcpy(precisionString[1],"MEDIUM");
strcpy(precisionString[2],"HIGH");

	return self;
}

// these will help to make proportionnal resize
- (float)widthFactor
{
	return (1.0/474.0)*NX_WIDTH(&bounds);
}
- (float)heightFactor
{
	return (1.0/226.0)*NX_WIDTH(&bounds);
}


- (void)resetRects
{
int i,points;
NXCoord xstep =7.0;
NXCoord xstart;
float ecart=0;
xstep=7.0;
[self resetInput];
switch (precision)
	{
	case XS3_DSA_PRECISION_LOW:		xstep=10.0;break;
	case XS3_DSA_PRECISION_MEDIUM:	xstep=7.0; break;
	case XS3_DSA_PRECISION_HIGH:	xstep=5.0; break;

	default:						xstep=10.0;break;
	}
NXSetRect(&myRects[0], 4.0, 5, NX_WIDTH(&bounds)-10, 44.0*[self heightFactor] );
points=pointCount;
if (stereo)
	{
	ecart=60;
	if (points>=80)
		ecart=40.0;
	if ( xstep*points>NX_WIDTH(&bounds)-2.0*8.0-ecart )
		xstep= (NX_WIDTH(&bounds) -ecart -2.0*8.0)/(float)points;
	xstart = (NX_WIDTH(&bounds) -ecart -xstep*points) / 2.0;
	for(i = 0; i < points; i++)
		{
		float offset=0.0;
		if (i>=points/2)
			offset=ecart;
		myRects[i].origin.x = xstart + offset + i * xstep;
		myRects2[i].origin.x =xstart + offset + i * xstep;
		myRects[i].origin.y = 5.0;//5
		myRects2[i].origin.y = 5.0;//5
		myRects[i].size.width = xstep;
		myRects2[i].size.width = xstep;
		}	
	}
else	
	{
	if ( xstep*points>=NX_WIDTH(&bounds) -2.0*8.0 )
		xstep= (NX_WIDTH(&bounds) -2.0*8.0)/(float)points;
	xstart = (NX_WIDTH(&bounds)-xstep*points) / 2.0;
	for(i = 0; i < points; i++)
		{	
		myRects[i].origin.x = xstart + i * xstep;
		myRects2[i].origin.x = xstart + i * xstep;
		myRects[i].origin.y = 5.0;//5
		myRects2[i].origin.y = 5.0;//5
		myRects[i].size.width = xstep;
		myRects2[i].size.width = xstep;
		}	
	}
}

- sizeTo:(NXCoord)width :(NXCoord)height
{
float dx_factor;
float dy_factor;
[super sizeTo:width :height];
dx_factor=(1.0/474.0)*NX_WIDTH(&bounds);
dy_factor=(1.0/226.0)*NX_WIDTH(&bounds);
// left
//NXSetRect(&dragRect[0],0.045*NX_WIDTH(&bounds),0.0929*NX_HEIGHT(&bounds),0.9*NX_WIDTH(&bounds),0.407*NX_HEIGHT(&bounds));

NXSetRect(&paletteRect1,
	80*[self widthFactor],
	107*[self heightFactor],
	120*[self widthFactor],
	40*[self heightFactor]);
NXSetRect(&paletteRect1,
	80*[self widthFactor],
	107*[self heightFactor],
	120*[self widthFactor],
	40*[self heightFactor]);
NXSetRect(&precisionRect,
	377*[self widthFactor],
	108*[self heightFactor],
	50*[self widthFactor],
	15*[self heightFactor]);
	return self;
}
- (void)clearScreen
{
NXRect r;
const int      mySides[] = {NX_YMIN, NX_XMAX, NX_YMAX, NX_XMIN, 
                        NX_YMIN, NX_XMAX,NX_YMAX, NX_XMIN};
const float    myGrays[] = {NX_WHITE, NX_WHITE, NX_DKGRAY, NX_DKGRAY, 
                        NX_LTGRAY, NX_LTGRAY,NX_DKGRAY, NX_DKGRAY};
NXRect  *aRect;//NX_LTGRAY
r=bounds;
aRect=&r;
PSsetgray(NX_BLACK);
PSrectfill(aRect->origin.x, aRect->origin.y, 
            aRect->size.width, aRect->size.height);

NXDrawTiledRects(aRect, (NXRect *)0, mySides, myGrays, 8);
}

// set stored data to 0,minimal value
- (void)resetInput
{
int i;
int points;
points=128;
for (i = 0 ; i < points; i++)
	maxima[i] = 0.0;
}

// scroll a view rect by the left
- (void)scrollRect:(NXRect *)aRect withHoryzontalOffset:(NXPoint *)aPoint
{
[self scrollRect:aRect by:aPoint];
}

// draw analysis given in one channel
- (void)drawFFTWithData
{
int i,max;
float ystart, yHeight;
NXRect r1;
NXPoint p;
NXCoord posX;
// scroll old values before drawing
p.x=-2;
p.y=0;
NXSetRect(&r1,4,4,NX_WIDTH(&bounds)-6,NX_HEIGHT(&bounds)-6);
[self scrollRect:&r1 withHoryzontalOffset:&p];
// Now we draw the new value from maxima variable
ystart= 4.0;
posX= NX_WIDTH(&bounds)-4;

	max=pointCount;
	yHeight= (NX_HEIGHT(&bounds)-4-2)/ (float)(max);

NXSetRect(&r1, posX, ystart, 2.0, yHeight);
for ( i = 0 ; i < max ; i++ )
	{
	float value;
	value=maxima[i];
	[self setLevelValue:value :palette1];
	NXRectFill(&r1);
	NX_Y(&r1)+=yHeight;
	}
}

// draw analysis given for both channels
- (void)drawStereoFFTWithData
{
int i;
int max;
NXRect r1,r2;
NXCoord ystart, xstep;
NXCoord dx;
NXCoord newValuePosX;
const float ECART=NX_HEIGHT(&bounds)*0.0884;// 20.0
NXPoint p;
p.x=-2;
p.y=0;
dx=(NX_HEIGHT(&bounds)-4-2-ECART)/2;// 100;
// scrolling
NXSetRect(&r1, 4.0, 4.0, bounds.size.width-2-2-2,dx);
[self scrollRect:&r1 withHoryzontalOffset:&p];
NXSetRect(&r1,4.0,4+dx+ECART,bounds.size.width-2-2-2,dx);
[self scrollRect:&r1 withHoryzontalOffset:&p];

//new maxima
ystart=2;
	max=pointCount;
xstep=((2*dx)-ECART-ystart)/(float)(max);
xstep=((2*dx))/(float)(max);
newValuePosX= NX_WIDTH(&bounds)-4;
NXSetRect(&r1, newValuePosX, ystart, 2.0, xstep);
NXSetRect(&r2, newValuePosX, ystart+dx+ECART, 2.0, xstep);
// max >>1 because we're in stereo mode
for (i=0;i< max>>1;i++)
	{
	float value;
	value=maxima[i+ (max >>1)];
	// left channel: bottom
	[self setLevelValue:value :palette1];
	NXRectFill(&r1);
	
	// right channel: top
	value=maxima[i];
	[self setLevelValue:value :palette1];
	NXRectFill(&r2);
	r1.origin.y+=xstep;
	r2.origin.y+=xstep;
	}
}

- (void)drawFFT:(float *)data 
{
int i;
// we store the new value in  maxima array
for (i=0;i<pointCount;i++)
	{
	maxima[i]=data[i];
	}

[self lockFocus];
if (stereo)
	[self drawStereoFFTWithData];
else 
	[self drawFFTWithData];
[self unlockFocus];
[window flushWindow];
}

// method called to give and draw the new data
- (void)updateWithDataInfo:(id)data
{
pointCount=[data dataCount];
if ((precision!=[data analysisPrecision])|| (stereo!=[data isDataStereo]))
	{
	[self choosePrecision:[data analysisPrecision]];
	stereo=[data isDataStereo];
	[self display];
	}
[self drawFFT:[data floatData] ];
}

// draw the scale reference in stereo mode
- (void)drawScaleForSpectre
{
int i;
float val;

	for (i=0;i<(int)(NX_WIDTH(&bounds)*0.2531);i++)
		{
		PSmoveto(NX_WIDTH(&bounds)*0.1687+i,NX_HEIGHT(&bounds)*0.4734);// ;107
		val=(float)i/(NX_WIDTH(&bounds)*0.2531);
		[self setLevelValue:val :palette1];
		PSrlineto(0.0 , NX_HEIGHT(&bounds)*0.0619);
		PSstroke();
		}
}
- (void)drawInfo
{
char buf[32];
float fontSize;
NXRect r={357,107,111,15};
NXSetRect(&r, NX_WIDTH(&bounds)*0.7531,
		NX_HEIGHT(&bounds)*0.4734,
		NX_WIDTH(&bounds)*0.2341,
		NX_HEIGHT(&bounds)*0.0663);
if (NX_HEIGHT(&r)>14)
	{
	fontSize=12;
	}
else
if (NX_HEIGHT(&r)<6)
	{
	return;
	}
else
	fontSize=NX_HEIGHT(&r);
switch (precision)
	{
	case 0:
	strcpy(buf,"[  ");
	strcat(buf, precisionString[0]);
	strcat(buf," ");
	break;
	case 1:
	strcpy(buf,"[");
	strcat(buf,precisionString[1]);
	break;
	case 2:
	strcpy(buf,"[  ");
	strcat(buf,precisionString[2]);
	strcat(buf," ");
	break;
	default: strcpy(buf,"[      "); break;
	}
strcat(buf,"]");

PSsetgray(NX_BLACK);
NXRectFill(&r);
if ([self shouldDrawColor])
	NXSetColor(NX_COLORGREEN);
else
	PSsetgray(NX_LTGRAY);
VPPSprintat(NX_WIDTH(&bounds)*0.7953,NX_HEIGHT(&bounds)*0.4867,fontSize, buf);
}

- (void)drawLargeSpectrum
{
NXRect r[2];
if (stereo)
	{
	NXSetRect(&r[0],3,90,NX_WIDTH(&bounds)-6,60);
	NXSetRect(&r[0],3,90+60,NX_WIDTH(&bounds)-6,60);
	}
else
	{
	NXSetRect(&r[0],3,90,NX_WIDTH(&bounds)-6,120);
	}

if (palette1==3)
	{
	NXSetRect(&r[0],3,90,NX_WIDTH(&bounds)-6,120);
	PSsetgray(NX_WHITE);
	NXRectFill(&r[0]);
	}
else
if ( (palette1!=3) )
	{
	NXSetRect(&r[0],3,90,NX_WIDTH(&bounds)-6,120);
	PSsetgray(NX_BLACK);
	NXRectFill(&r[0]);
	}

}

- drawSelf:(const NXRect *)rects :(int)rectCount
{
[self clearScreen];
// scales
[self drawLargeSpectrum];
if (stereo)
	{
	[self drawInfo];
	[self drawScaleForSpectre];
	}
    return self;
}

- mouseDown:(NXEvent *)theEvent
{
NXPoint pt;
// no change possible in mono mode , for this release...
if (!stereo)
	return self;
pt=theEvent->location;
[self convertPoint:&pt fromView:nil];
if (NXPointInRect(&pt,&paletteRect1) )
	selection_mode=1;

if (NXPointInRect(&pt,&precisionRect) )
	selection_mode=10;
	return self;
}

- mouseUp:(NXEvent *)theEvent
{
NXPoint pt;
// no change possible in mono mode , for this release...
if (!stereo)
	return self;
pt=theEvent->location;
[self convertPoint:&pt fromView:nil];
if ( (NXPointInRect(&pt,&paletteRect1)) &&(selection_mode==1) )
	{
	if (++palette1>2) palette1=0;
	if (palette1==3) 
		{
		//Inversed;
		[self display];
		}		
	else
		{
		if (palette1==0)
			{
			//Normal ,last was inversed so we display all the view;
			[self display];
			}
		else
			if (stereo)
				{// update scale display only in stereo mode
				[self lockFocus];
				[self drawScaleForSpectre];
				[self unlockFocus];
				[window flushWindow];
				}
		}
	}
if ((NXPointInRect(&pt,&precisionRect)) &&(selection_mode==10) )
	{
	precision++;
	if (precision>XS3_DSA_PRECISION_HIGH) precision=XS3_DSA_PRECISION_LOW;
	[self choosePrecision:precision];
	[self lockFocus];
	[self drawInfo];
	[self unlockFocus];
	[self resetRects];
	}
selection_mode=0;
	return self;
}

- (void)chooseStereo:(BOOL)flag
{
if (stereo==flag) return;
stereo=flag;
[self resetRects];
// draw type changed : view cleared
[self lockFocus];
[self clearScreen];
if (stereo)
	{
	[self drawInfo];
	[self drawScaleForSpectre];
	}
[self unlockFocus];
}

- (void)choosePrecision:(int)aPrecision
{
precision=aPrecision;
[self resetRects];
[VoicePrinter setPrecision:aPrecision];
}


- free
{
msgTable=nil;
	return [super free];
}
@end
